package com.ggreiff.gel.tag.javascript;

import com.niku.union.gel.XMLOutputBuffer;
import org.apache.commons.jelly.*;
import org.apache.commons.jelly.tags.core.ArgTagParent;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ContextFactory;
import org.mozilla.javascript.EcmaError;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;


/**
 * Created with IntelliJ IDEA.
 * User: ggreiff
 * Date: 8/18/14
 * Time: 8:54 AM
 */
public class JavascriptTag extends TagSupport implements ArgTagParent {

    private static final Log log = LogFactory.getLog(JavascriptTag.class);

    private static final String REFERENCE_ERROR = "ReferenceError";

    private String mainScript;

    private String functionScript;

    private Context jsContext;

    private Scriptable scope;

    private List paramTypes = new ArrayList();
    private List paramValues = new ArrayList();

    public void addArgument(Class type, Object value) {
        this.paramTypes.add(type);
        this.paramValues.add(value);
    }

    @Override
    public void doTag(XMLOutput xmlOutput) throws MissingAttributeException, JellyTagException {
        try {
        XMLOutputBuffer wrapper = new XMLOutputBuffer();
        invokeBody(wrapper);

        Object[] values = this.paramValues.toArray();

        for (Object value : values) {
            ScriptType scriptType = (ScriptType) value;
            if (scriptType.getScriptPart() == ScriptPart.MAIN) this.mainScript = scriptType.getScript();
            if (scriptType.getScriptPart() == ScriptPart.FUNCTION) this.functionScript = scriptType.getScript();
        }


        // See if there was a mainScriptTag.
        if (this.mainScript == null) {
            this.mainScript = getBodyText();
        }

        if (this.mainScript == null || this.mainScript.length() == 0) {
            throw new JellyTagException("The 'mainScript' body can not be empty");
        }

        JsContextMap jsContextMap = new JsContextMap();

        Iterator iterator = this.context.getVariableNames();
        while (iterator.hasNext()) {
            String key = (String) iterator.next();
            jsContextMap.putJsValue(key, JsValue.create(this.context.getVariable(key)));
        }

        if (this.functionScript != null && this.functionScript.length() > 0) addFunctions(this.functionScript);

        evaluate(this.mainScript, jsContextMap);

        ScriptableObject scopeObject = (ScriptableObject) scope;

        Object[] ids = scopeObject.getIds();
        for (Object id : ids) {
            Object findJsValue = scopeObject.get(id.toString(), scope);
            Object findGelVariable = this.context.findVariable(id.toString());
            Object findGelValue = null;

            if (findGelVariable != null) findGelValue = this.context.getVariable(id.toString());
            if (findGelVariable == null || findGelValue != findJsValue) {
                this.context.setVariable(id.toString(), scopeObject.get(id.toString(), scope));
            }
        }
        } catch (Exception ex) {
            throw ex;
        }
        finally {
            Context.exit();
        }

    }

    public JsValue evaluate(String expression, JsContextMap table) {
        List<JsVariable> variables = table.getJsVariables();
        return evaluateExpression(expression, variables);
    }

    private void addFunctions(String functionScript) {
        if (this.jsContext == null) jsContext = ContextFactory.getGlobal().enterContext();
        this.scope = getScope(jsContext, null);
        jsContext.evaluateString(this.scope, functionScript, "", 1, null);
    }

    private JsValue evaluateExpression(String expression, List<JsVariable> variables) {

        try {
            if (this.jsContext == null) this.jsContext = ContextFactory.getGlobal().enterContext();
            this.jsContext = ContextFactory.getGlobal().enterContext();
            this.scope = getScope(this.jsContext, variables);
            Object result = this.jsContext.evaluateString(this.scope, expression, "", 1, null);
            return JsValue.create(result);
        } catch (EcmaError error) {
            return handleException(error, variables);
        }
    }

    private Scriptable getScope(Context context, List<JsVariable> variables) {
        if (this.scope == null) this.scope = context.initStandardObjects();
        if (variables != null) putJavaVariablesIntoEcmaScope(this.scope, variables);
        return this.scope;
    }

    private static void putJavaVariablesIntoEcmaScope(Scriptable scope,
                                                      List<JsVariable> variables) {

        for (JsVariable variable : variables) {
            putJavaVariableIntoEcmaScope(scope, variable);
        }
    }

    private static void putJavaVariableIntoEcmaScope(Scriptable scope,
                                                     JsVariable variable) {

        String variableName = variable.getName();
        JsValue ecmaValue = variable.getValue();
        Object javaValue = ecmaValue.getValue();

        Object wrappedValue = Context.javaToJS(javaValue, scope);
        ScriptableObject.putProperty(scope, variableName, wrappedValue);
    }

    private static JsValue handleException(EcmaError error,
                                           List<JsVariable> variables) {

        if (REFERENCE_ERROR.equals(error.getName())) {
            throw new IllegalArgumentException("I couldn't resolve some "
                    + "variable on expression with vars "
                    + Arrays.toString(variables.toArray()), error);
        }
        throw error;
    }


}
