package com.ggreiff.gel.tag.xls;

import org.apache.commons.jelly.JellyTagException;
import org.apache.commons.jelly.TagSupport;
import org.apache.commons.jelly.XMLOutput;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFCellUtil;

import java.math.BigDecimal;

/**
 * Created with IntelliJ IDEA.
 * User: ggreiff
 * Date: 3/10/14
 * Time: 9:41 AM
 */
public class XlsCellTag extends TagSupport {
    private static final Log log = LogFactory.getLog(XlsCellTag.class);

    private String _var = "";
    private String _value = "";
    private String _style = "";
    private String _dataType = "";
    private int _row = -1;
    private int _column = -1;
    private HSSFCell _hssfCell;
    private HSSFRow _hssfRow;
    private HSSFSheet _hssfSheet;


    public XlsCellTag() {
    }

    public void doTag(XMLOutput output) throws JellyTagException {

        if (this._row < 0 || this._column < 0) {
            throw new JellyTagException("Cell needs a valid row/column number");
        }

        if (this._hssfSheet == null ) {
            if (this.context.getVariables().containsKey("org.apache.poi.hssf.usermodel.HSSFSheet"))
                this._hssfSheet = (HSSFSheet) this.context.getVariable("org.apache.poi.hssf.usermodel.HSSFSheet");
        }

        if (this._hssfSheet == null) {
            log.error("Could not load worksheet");
            throw new JellyTagException("Cell needs a worksheet to execute on");
        }

        if (this._row > this._hssfSheet.getLastRowNum()) {
            int loopCnt = this._row - this._hssfSheet.getLastRowNum();
            for (int i = this._hssfSheet.getLastRowNum(); i <= loopCnt; i++) {
                this._hssfRow = this._hssfSheet.createRow(i);
            }
        }

        this._hssfCell = HSSFCellUtil.getCell(this._hssfRow, _column);
        if (this._value != null && !this._value.isEmpty()) {

            Boolean isSet = false;
            if (_dataType.toLowerCase().equals("currency")){
                Double currency = parseExeclNumber(this._value);
                if (currency != null){
                    this._hssfCell.setCellValue(currency);
                    isSet = true;
                }
            }

            if (_dataType.toLowerCase().equals("datetime")){

            }

            if (_dataType.toLowerCase().equals("logical")){
                if (this._value.toUpperCase().equals("TRUE") || this._value.toUpperCase().equals("FALSE")){
                    this._hssfCell.setCellFormula(this._value.toUpperCase());
                    isSet = true;
                }
            }

            if (_dataType.toLowerCase().equals("number")){
                Double number = parseExeclNumber(this._value);
                if (number != null){
                    this._hssfCell.setCellValue(number);
                    isSet = true;
                }
            }

            if (_dataType.toLowerCase().equals("text")){

            }



            this._hssfCell.setCellValue(this._value);
        }



        this.context.setVariable(this._var == null || this._var.isEmpty() ? "org.apache.poi.hssf.usermodel.HSSFCell" : this._var, this._hssfCell);
        invokeBody(output);
    }

    public Double parseExeclNumber(String data) {
        Double retVal = null;
        try {
            retVal = Double.parseDouble(data);
        } catch (Exception ex) { }
        return retVal;
    }

    public void setVar(String var) {
        this._var = var;
    }

    public void setValue(String value) {
        this._value = value;
    }

    public void setSheet(HSSFSheet sheet) {
        this._hssfSheet = sheet;
    }

    public void setRow(String row) {
        try {
            this._row = Integer.parseInt(row) - 1;
        } catch (Exception ex) {
            log.error("Row need to be a postive integer greater than 0");
            this._row = -1;
        }
    }

    public void setColumn(String column) {
        try {
            this._column = Integer.parseInt(column) - 1;
        } catch (Exception ex) {
            log.error("Column need to be a postive integer greater than 0");
            this._column = -1;
        }
    }

    public void setStyle(String style){
        this._style = style;
    }

    public void setDataType(String dataType){
        this._dataType = dataType;
    }
}
