package com.ggreiff.gel.tag.xls;

import org.apache.commons.jelly.JellyTagException;
import org.apache.commons.jelly.TagSupport;
import org.apache.commons.jelly.XMLOutput;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;

/**
 * Created with IntelliJ IDEA.
 * User: ggreiff
 * Date: 3/10/14
 * Time: 11:41 AM
 */
public class XlsSaveTag extends TagSupport {
    private static final Log log = LogFactory.getLog(XlsSaveTag.class);

    private String _fileName = "";
    private String _var = "";
    private Boolean _overWrite = false;
    private HSSFWorkbook _hssfWorkBook;


    public XlsSaveTag() {
    }

    public void doTag(XMLOutput output) throws JellyTagException {

        if (this._fileName.isEmpty()) {
            if (this.context.getVariables().containsKey("com.ggreiff.gel.tag.xls.XlsOpenTag.fileName")) {
                this._fileName = (String) this.context.getVariable("com.ggreiff.gel.tag.xls.XlsOpenTag.fileName");
            }
        }

        if (this._fileName.isEmpty()) {
            log.error("Could not save xls file");
            throw new JellyTagException("Unable to load the xls file need a fileName");
        }

        File f = new File(this._fileName);
        if (f.isDirectory()) {
            log.error("FileName is pointing to directory instead of xls file.");
            throw new JellyTagException("Unable to save the xls file need a valid fileName");
        }

        if (f.exists() && !this._overWrite) {
            log.error("Xls file exist and overWrite is set to false;");
            throw new JellyTagException("Unable to overWrite existing xls file");
        }

        if (this._hssfWorkBook == null) {
            if (this.context.getVariables().containsKey("org.apache.poi.hssf.usermodel.HSSFWorkbook")) {
                this._hssfWorkBook = (HSSFWorkbook) this.context.getVariable("org.apache.poi.hssf.usermodel.HSSFWorkbook");
            }
            if (this._hssfWorkBook == null) {
                log.error("Could not save xls file");
                throw new JellyTagException("Save needs a workbook to execute on");
            }
        }


        try {
            FileOutputStream out = new FileOutputStream(f, false);
            this._hssfWorkBook.write(out);
            out.close();

        } catch (Exception ex) {
            log.error("Error in saving xls file");
            throw new JellyTagException("Error in saving xls file\n" + ex.getMessage());
        }
    }

    public void setFileName(String fileName) {
        this._fileName = fileName;
    }

    public void setVar(String var) {
        this._var = var;
    }

    public void setOverWrite(String overWrite) {
        this._overWrite = overWrite.toLowerCase().matches("yes");
    }

    public void setWorkBook(HSSFWorkbook workbook){
        this._hssfWorkBook = workbook;
    }
}


